from .preview import Preview
from typing import List
import os
import logging

class BookPreview(Preview):
    def __init__(self, pad):
        super().__init__(pad)
        self.content: List[str] = []

    def update(self, row: int, book_dir):
        super().update(row)
        self.content = map(lambda f: os.path.splitext(f)[0],
                sorted(
                    filter(lambda f: os.path.splitext(f)[1] == '.vmnt',
                        os.listdir(book_dir)),
                    key=lambda f: os.stat(os.path.join(book_dir, f)).st_mtime_ns, reverse=True)[:self.pad.getmaxyx()[0] - 1])




#!/usr/bin/env python3.10

from .bookview import BookView
from .noteview import NoteView
from .tableview import TableView
from .config import get_config
from .exceptions import ExitException, OpenBookException, CloseBookException, EditNoteException

import sys
import os
import curses
import logging
import subprocess as sp

from typing import List, Any

CONFIG = get_config(os.path.expanduser('~/.config/vimnote'))

__help__ = '''vimnote - a vim-based TUI notetaking application
USAGE: vimnote              - launch into vimnote
       vimnote [book name]  - launch into vimnote and open a book immediately'''

class suspend_curses():
    # see https://stackoverflow.com/a/20769213/16834825 for justification/implementation
    def __enter__(self):
        curses.endwin()

    def __exit__(self, exc_type, exc_val, tb):
        newscr = curses.initscr()
        newscr.clear()
        newscr.refresh()
        curses.doupdate()

def real_main(stdscr, book):
    stdscr.clear()
    curses.curs_set(False)
    curses.use_default_colors()
    stdscr.refresh() # required for some reason, otherwise doesn't refresh until first keypress

    # logging.basicConfig(filename='log.log', level=logging.DEBUG)

    if book is None:
        view = BookView(stdscr, CONFIG)
    else:
        view = NoteView(stdscr, CONFIG, book)

    while True:
        view.draw()

        # break on keyboard interrupt
        try:
            key = stdscr.getch()
        except KeyboardInterrupt:
            break

        # if the terminal has been resized, clear stdscr and pad to avoid solitaire-ing
        if key == curses.KEY_RESIZE:
            stdscr.clear()
            view.pad.clear()
        stdscr.refresh()

        # else let the view handle it
        try: view.handle_keypress(key)
        except ExitException: break
        except OpenBookException as e:
            stdscr.clear()
            stdscr.refresh()
            view = NoteView(stdscr, CONFIG, e.title)
        except CloseBookException:
            stdscr.clear()
            stdscr.refresh()
            view = BookView(stdscr, CONFIG)
        except EditNoteException as e:
            with suspend_curses():
                sp.run(['vim', os.path.join(CONFIG['notedir'], e.book, e.title + '.vmnt')])
            view = NoteView(stdscr, CONFIG, e.book)

def main():
    os.environ['ESCDELAY'] = '25' # avoid long delay after hitting escape
    book = None
    if len(sys.argv) > 1:
        book = ' '.join(sys.argv[1:])
    if book in ('--help', '-h'):
        print(__help__)
    else:
        curses.wrapper(real_main, book)

if __name__ == '__main__':
    main()



import curses
from .exceptions import ExitException

class TextBox:
    def __init__(self, prompt: str = '', size: int = 0):
        self.prompt = prompt
        self.size = size

        self.noscroll_size = 0.9

        self.reset()

    def reset(self):
        self.text = ''
        self.scroll = 0
        self.cursor_pos = 0

    def move_cursor_pos(self, pos: int):
        self.cursor_pos = pos
        right_cutoff_size = round(self.size * self.noscroll_size)
        self.scroll = self.cursor_pos - right_cutoff_size
        self.scroll = max(0, min(self.scroll, len(self.prompt) + len(self.text) - round(self.size * self.noscroll_size)))

    def draw(self, win, y: int, x: int, color_pair, left_offset: int = 0):
        # this f-string is basically just:
        # - left padding
        # - main content, cropped for scrolling
        # - right padding
        win.addstr(y, x,
                f'{" " * left_offset}{f"{self.prompt}{self.text}"[self.scroll : self.scroll + self.size]}{" " * (self.size + self.scroll - len(self.text) - len(self.prompt))}',
                color_pair)
        win.move(y, x + left_offset + len(self.prompt) + self.cursor_pos - self.scroll)

    def handle_keypress(self, key: int) -> int:
        match key:
            case curses.KEY_BACKSPACE | 127 | 8:
                self.text = self.text[:self.cursor_pos - 1] + self.text[self.cursor_pos:]
                self.move_cursor_pos(max(self.cursor_pos - 1, 0))
            case curses.KEY_DC:
                self.text = self.text[:self.cursor_pos] + self.text[self.cursor_pos + 1:]
                self.move_cursor_pos(min(self.cursor_pos, len(self.text)))
            case curses.KEY_LEFT:
                self.move_cursor_pos(max(self.cursor_pos - 1, 0))
            case curses.KEY_RIGHT:
                self.move_cursor_pos(min(self.cursor_pos + 1, len(self.text)))
            case curses.KEY_UP:
                self.move_cursor_pos(0)
            case curses.KEY_DOWN:
                self.move_cursor_pos(len(self.text))
            case curses.KEY_ENTER | 10:
                return 0
            case 27: # escape
                self.move_cursor_pos(0)
                self.text = ''
                return 1
            case 4:
                raise ExitException
            case _:
                char = chr(key)
                if char.isprintable():
                    self.text = self.text = self.text[:self.cursor_pos] + char + self.text[self.cursor_pos:]
                    self.move_cursor_pos(self.cursor_pos + 1)
        return -1



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['vimnote']

package_data = \
{'': ['*']}

entry_points = \
{'console_scripts': ['vimnote = vimnote:main']}

setup_kwargs = {
    'name': 'vimnote',
    'version': '1.0.4',
    'description': 'a vim-based TUI notetaking application',
    'long_description': None,
    'author': 'maxcutlyp',
    'author_email': 'max@maxcutlyp.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'entry_points': entry_points,
    'python_requires': '>=3.10',
}


setup(**setup_kwargs)



class Preview:
    def __init__(self, pad):
        self.pad = pad
        self.internal_row = 0 # so we know whether to update or not

    def update(self, row: int, *args):
        self.internal_row = row

    def draw(self):
        self.pad.clear()
        width = self.pad.getmaxyx()[1]
        self.pad.addstr(0, 0, f'{"─" * width}')
        for row,line in enumerate(self.content):
            self.pad.addstr(row + 1, 1, line)



from .exceptions import ExitException
import curses
from typing import List

class DeleteDialog:
    def __init__(self, text: List[str]):
        self.text = text

        curses.init_pair(4, curses.COLOR_RED, -1) # for delete button unhighlighted
        curses.init_pair(5, curses.COLOR_WHITE, curses.COLOR_RED) # for delete button highlighted
        curses.init_pair(6, curses.COLOR_BLACK, curses.COLOR_CYAN) # for cancel button highlighted

        # (text, unhighlighted color, highlighted color)
        self.left_button = ('Cancel', curses.color_pair(0), curses.color_pair(6))
        self.right_button = ('Delete', curses.color_pair(4), curses.color_pair(5))

        self.l_highlighted = True # only two states (left or right) so bool can be used
        
    def draw(self, stdscr):
        height, width = stdscr.getmaxyx()
        dialog_width = max(max(len(line) for line in self.text), len(self.left_button[0]) + len(self.right_button[0]) + 8) + 2
        dialog_height = len(self.text) + 2
        start_x = round((width - dialog_width)/2)
        start_y = round(height * 0.4)

        stdscr.addstr(start_y - 1, start_x, f'╭{"─" * dialog_width}╮')
        for i,line in enumerate(self.text):
            stdscr.addstr(start_y + i, start_x, f'│{line:^{dialog_width}.{dialog_width}}│')
        stdscr.addstr(start_y + len(self.text), start_x, f'│{" " * dialog_width}│')
        stdscr.addstr(start_y + len(self.text) + 1, start_x, f'│{" " * dialog_width}│')

        stdscr.addstr(start_y + len(self.text) + 1, round(width/2) - len(self.left_button[0]) - 2,
                f' {self.left_button[0]} ', self.left_button[2] if self.l_highlighted else self.left_button[1])
        stdscr.addstr(start_y + len(self.text) + 1, round(width/2) + 2,
                f' {self.right_button[0]} ', self.right_button[1] if self.l_highlighted else self.right_button[2])
        stdscr.addstr(start_y + len(self.text) + 2, start_x, f'╰{"─" * dialog_width}╯')
    
    def handle_keypress(self, key) -> int:
        match (key, chr(key)):
            case (_, ('h' | 'k')) | (curses.KEY_LEFT, _):
                self.l_highlighted = True
            case (_, ('l' | 'j')) | (curses.KEY_RIGHT, _):
                self.l_highlighted = False
            case (4, _) | (_, 'q'):
                raise ExitException
            case ((curses.KEY_ENTER | 10), _) | (_, ' '):
                return 1 if self.l_highlighted else 0
            case (27, _): # escape
                return 1
        return -1
